//
//  TALocationService.m
//  JoinUpTaxiPassenger
//
//  Created by Christopher Alford on 05/06/2013.
//
//

#import "DOLocationService.h"
#import "DOStreetSearchCall.h"

#define ANNOTATION_REGION_PAD_FACTOR 1.25
#define MAX_DEGREES_ARC 30.0
#define MINIMUM_ZOOM_ARC 0.01

static int kMinimumDeltaDistance = 500.0f;

static DOLocationService* shared = nil;

@interface DOLocationService () <CLLocationManagerDelegate>

@property (strong, nonatomic) CLLocationManager* locationManager;
@property (strong, nonatomic) NSDictionary *cityBounds;
@property (strong, nonatomic) DOStreetSearchCall *streetSearchCall;

@end

@implementation DOLocationService

@synthesize address = _address, city = _city;

+ (DOLocationService *)sharedInstance {
    if(!shared){
        static dispatch_once_t pred;    // Lock
        dispatch_once(&pred, ^{         // This code is called at most once per app
            shared = [[DOLocationService alloc] init];
        });
    }
    return shared;
}

- (id)init {
    self = [super init];
    if(self){
        return self;
    }
    return nil;
}

- (CLLocationManager *)locationManager
{
    if(!_locationManager) {
        _locationManager = [[CLLocationManager alloc] init];
        [_locationManager setDistanceFilter:kCLDistanceFilterNone];
        [_locationManager setDesiredAccuracy:kCLLocationAccuracyBest];
        [_locationManager setDelegate:self];
        [_locationManager startUpdatingLocation];
    }
    return _locationManager;
}

- (NSDictionary *)cityBounds
{
    if(!_cityBounds) {
        NSMutableDictionary *newCityBounds = [[NSMutableDictionary alloc]init];
        
        NSArray *barcelona = @[[[CLLocation alloc]initWithLatitude:41.391671 longitude:2.177609],
                               [[CLLocation alloc]initWithLatitude:41.375378 longitude:2.070831],
                               [[CLLocation alloc]initWithLatitude:41.428528 longitude:2.173038],
                               [[CLLocation alloc]initWithLatitude:41.429018 longitude:2.237203]
                               ];
        
        [newCityBounds setValue:barcelona forKey:@"Barcelona"];
        
        _cityBounds = [[NSDictionary alloc]initWithDictionary:newCityBounds copyItems:YES];
        newCityBounds = nil;
    }
    return _cityBounds;
}

#pragma mark - Public methods

- (void)stopUpdating
{
    //[self.locationManager stopMonitoringSignificantLocationChanges];
    [self.locationManager stopUpdatingLocation];
}

- (void)resumeUpdating
{
    //[self.locationManager startMonitoringSignificantLocationChanges];
    [self.locationManager startUpdatingLocation];
}

- (void)addressForCoordinate:(CLLocationCoordinate2D) coordinate
{
    /*
    if(self.streetSearchCall) {
        [self.streetSearchCall cancel];
    }
     */
    
    self.streetSearchCall = [[DOStreetSearchCall alloc] initWithBaseURL:@"BaseGeocodeURL"];
    self.streetSearchCall.coordinate = coordinate;
    
    NSString * languageX = [[NSLocale preferredLanguages] objectAtIndex:0];
    
    // Only use the first 2 chars from the language string
    NSString *language = [languageX substringToIndex:2];
    self.streetSearchCall.urlString = [NSString stringWithFormat:@"sensor=false&language=%@&latlng=%f,%f", language, coordinate.latitude, coordinate.longitude];
    self.streetSearchCall.delegate = self;
    self.isUpdatingAddress = YES;
}

- (MKCoordinateRegion)regionForCityNamed:(NSString *)cityName
{
    NSArray *locations = [self.cityBounds objectForKey:cityName];
    
    MKMapPoint mapPoints[[locations count]];
    
    int i = 0;
    
    for(CLLocation *location in locations) {
        CLLocationCoordinate2D coordinate = location.coordinate;
        mapPoints[i++] = MKMapPointForCoordinate(coordinate);
    }
    
    // For our case, ensure the users current location is in the array, otherwise zooms out to whole world??? Dodgy code!
    CLLocationCoordinate2D current = CLLocationCoordinate2DMake(self.currentLocation.coordinate.latitude, self.currentLocation.coordinate.longitude);
    mapPoints[i] = MKMapPointForCoordinate(current);
    
    //create MKMapRect from array of MKMapPoint
    MKMapRect mapRect = [[MKPolygon polygonWithPoints:mapPoints count:[locations count]] boundingMapRect];
    //convert MKCoordinateRegion from MKMapRect
    MKCoordinateRegion region = MKCoordinateRegionForMapRect(mapRect);
    
    //add padding so pins aren't scrunched on the edges
    region.span.latitudeDelta  *= ANNOTATION_REGION_PAD_FACTOR;
    region.span.longitudeDelta *= ANNOTATION_REGION_PAD_FACTOR;
    //but padding can't be bigger than the world
    if( region.span.latitudeDelta > MAX_DEGREES_ARC ) { region.span.latitudeDelta  = MAX_DEGREES_ARC; }
    if( region.span.longitudeDelta > MAX_DEGREES_ARC ){ region.span.longitudeDelta = MAX_DEGREES_ARC; }
    
    //and don't zoom in stupid-close on small samples
    if( region.span.latitudeDelta  < MINIMUM_ZOOM_ARC ) { region.span.latitudeDelta  = MINIMUM_ZOOM_ARC; }
    if( region.span.longitudeDelta < MINIMUM_ZOOM_ARC ) { region.span.longitudeDelta = MINIMUM_ZOOM_ARC; }
    //and if there is a sample of 1 we want the max zoom-in instead of max zoom-out
    if( [locations count] == 1 )
    {
        region.span.latitudeDelta = MINIMUM_ZOOM_ARC;
        region.span.longitudeDelta = MINIMUM_ZOOM_ARC;
    }
    return region;
}

- (MKCoordinateRegion)regionToFitAnnotations:(NSArray *)annotations
{
    
    //convert NSArray of id <MKAnnotation> into an MKCoordinateRegion that can be used to set the map size
    int i = 0;
    int count = [annotations count];
    MKMapPoint points[count]; //C array of MKMapPoint struct
    
    for(id<MKAnnotation> annotation in annotations) {
        CLLocationCoordinate2D coordinate = annotation.coordinate;
        points[i++] = MKMapPointForCoordinate(coordinate);
    }
    
    // For our case, ensure the users current location is in the array, otherwise zooms out to whole world??? Dodgy code!
    CLLocationCoordinate2D current = CLLocationCoordinate2DMake(self.currentLocation.coordinate.latitude, self.currentLocation.coordinate.longitude);
    points[i] = MKMapPointForCoordinate(current);
    
    //create MKMapRect from array of MKMapPoint
    MKMapRect mapRect = [[MKPolygon polygonWithPoints:points count:count] boundingMapRect];
    //convert MKCoordinateRegion from MKMapRect
    MKCoordinateRegion region = MKCoordinateRegionForMapRect(mapRect);
    
    //add padding so pins aren't scrunched on the edges
    region.span.latitudeDelta  *= ANNOTATION_REGION_PAD_FACTOR;
    region.span.longitudeDelta *= ANNOTATION_REGION_PAD_FACTOR;
    //but padding can't be bigger than the world
    if( region.span.latitudeDelta > MAX_DEGREES_ARC ) { region.span.latitudeDelta  = MAX_DEGREES_ARC; }
    if( region.span.longitudeDelta > MAX_DEGREES_ARC ){ region.span.longitudeDelta = MAX_DEGREES_ARC; }
    
    //and don't zoom in stupid-close on small samples
    if( region.span.latitudeDelta  < MINIMUM_ZOOM_ARC ) { region.span.latitudeDelta  = MINIMUM_ZOOM_ARC; }
    if( region.span.longitudeDelta < MINIMUM_ZOOM_ARC ) { region.span.longitudeDelta = MINIMUM_ZOOM_ARC; }
    //and if there is a sample of 1 we want the max zoom-in instead of max zoom-out
    if( count == 1 )
    {
        region.span.latitudeDelta = MINIMUM_ZOOM_ARC;
        region.span.longitudeDelta = MINIMUM_ZOOM_ARC;
    }
    return region;
}

- (void)cancelUpdatingAddress
{
    //[self.streetSearchCall cancel];
    self.streetSearchCall = nil;
    self.isUpdatingAddress = NO;
}

#pragma mark - delegate

-(void)locationManager:(CLLocationManager *)manager
   didUpdateToLocation:(CLLocation *)newLocation
          fromLocation:(CLLocation *)oldLocation
{
    float distanceMoved = [newLocation distanceFromLocation:self.currentLocation];
    
    if (!self.currentLocation) {
        self.currentLocation = newLocation;
        [self.delegate newPositionWithLocation:self.currentLocation];
        return;
    }
    
    if ((distanceMoved > kMinimumDeltaDistance) && ([newLocation.timestamp timeIntervalSinceDate:self.currentLocation.timestamp] > 10.0)) {
        // Only update once every 10 seconds maximum, unless the location is not accurate
        if(self.currentLocation.horizontalAccuracy > newLocation.horizontalAccuracy){
            self.currentLocation = newLocation;
            [self.delegate newPositionWithLocation:self.currentLocation];
        }
    }
}

#pragma mark - DOStreetSearchCall delegate

- (void)fetchStarted {}
- (void)fetchFailedWithError:(NSError *)error
{
    _address = nil;
    _city = nil;
    self.streetSearchCall = nil;
}
- (void)fetchReportedMessage:(NSString *)message  {}

- (void)fetchCompleted
{
    self.isUpdatingAddress = NO;
    if (self.streetSearchCall.foundStreet && ![self.streetSearchCall.foundStreet isEqualToString:@"(null) "] && ![self.streetSearchCall.foundStreet isEqualToString:@"unknownStreet"]) {
        
        _address = self.streetSearchCall.foundStreet;
        _city = self.streetSearchCall.cityName;
        
        NSDictionary *newGeoLocation = [NSDictionary dictionaryWithObjects:@[_address, _city] forKeys:@[@"address", @"city"]];
        [self.delegate newGeoLocationWithDictionary:newGeoLocation];
    }
    self.streetSearchCall = nil;
}



@end
